@@ -1,7 +1,7 @@
 from scNodes.core import settings
 from scNodes.core import config as cfg
 from scNodes.core.opengl_classes import *
-from PIL import Image
+import tifffile
 import mrcfile
 from copy import copy, deepcopy
 import datetime
@@ -35,8 +35,12 @@ class CLEMFrame:
         self.n_slices = 1
         self.current_slice = 0
         self.extension = ""
+        self.particle_groups = list()
+        self.particle_groups.append(ParticleGroup())
+        self.current_particle_group = self.particle_groups[0]
         # transform parameters
         self.pixel_size = cfg.ce_default_pixel_size  # pixel size in nm
+        self.pixel_size_z = cfg.ce_default_pixel_size
         self.pivot_point = np.zeros(2)  # pivot point for rotation and scaling of this particular image. can be moved by the user. In _local coordinates_, i.e. relative to where the frame itself is positioned.
         self.transform = Transform()
         self.flip_h = False
@@ -84,9 +88,16 @@ class CLEMFrame:
         if horizontally:
             self.flip_h = not self.flip_h
             self.data = np.flip(self.data, axis=1)
+            # flip particles
+            for group in self.particle_groups:
+                for i in range(len(group.coordinates)):
+                    group.coordinates[i][0] = self.width - group.coordinates[i][0]
         else:
             self.flip_v = not self.flip_v
             self.data = np.flip(self.data, axis=0)
+            for group in self.particle_groups:
+                for i in range(len(group.coordinates)):
+                    group.coordinates[i][1] = -(self.height + group.coordinates[i][1])
         self.update_image_texture(compute_histogram=False)
 
     def setup_opengl_objects(self):
@@ -117,9 +128,7 @@ class CLEMFrame:
         # else, load slice and update texture.
         self.current_slice = requested_slice
         if self.extension == ".tiff" or self.extension == ".tif":
-            data = Image.open(self.path)
-            data.seek(self.current_slice)
-            self.data = np.asarray(data)
+            self.data = tifffile.imread(self.path, key=self.current_slice).astype(float)
             if cfg.ce_flip_on_load:
                 self.data = np.flip(self.data, axis=1)
             if self.flip_h:
@@ -131,10 +140,12 @@ class CLEMFrame:
             mrc = mrcfile.mmap(self.path, mode="r")
             self.n_slices = mrc.data.shape[0]
             self.current_slice = min([self.current_slice, self.n_slices - 1])
-            if cfg.ce_flip_mrc_on_load:
-                self.data = mrc.data[:, self.current_slice, :]
-            else:
-                self.data = mrc.data[self.current_slice, :, :]
+            self.data = mrc.data[self.current_slice, :, :]
+            # type conversions
+            if type(self.data[0, 0]) == np.int8:
+                self.data = self.data.astype(np.uint8, copy=False)
+            elif type(self.data[0, 0]) == np.int16:
+                self.data = self.data.astype(np.uint16, copy=False)
             if self.flip_h:
                 self.data = np.flip(self.data, axis=1)
             if self.flip_v:
@@ -142,31 +153,36 @@ class CLEMFrame:
             self.update_image_texture()
 
     def translate(self, translation):
-        self.pivot_point[0] += translation[0]
-        self.pivot_point[1] += translation[1]
-        self.transform.translation[0] += translation[0]
-        self.transform.translation[1] += translation[1]
+        for frame in self.list_all_children(include_self=True):
+            frame.pivot_point[0] += translation[0]
+            frame.pivot_point[1] += translation[1]
+            frame.transform.translation[0] += translation[0]
+            frame.transform.translation[1] += translation[1]
 
     def pivoted_rotation(self, pivot, angle):
-        self.transform.rotation += angle
-        p = np.matrix([self.transform.translation[0] - pivot[0], self.transform.translation[1] - pivot[1]]).T
-        rotation_mat = np.identity(2)
-        _cos = np.cos(angle / 180.0 * np.pi)
-        _sin = np.sin(angle / 180.0 * np.pi)
-        rotation_mat[0, 0] = _cos
-        rotation_mat[1, 0] = _sin
-        rotation_mat[0, 1] = -_sin
-        rotation_mat[1, 1] = _cos
-        delta_translation = rotation_mat * p - p
-        delta_translation = [float(delta_translation[0]), float(delta_translation[1])]
-        self.transform.translation[0] += delta_translation[0]
-        self.transform.translation[1] += delta_translation[1]
+        for frame in self.list_all_children(include_self=True):
+            frame.transform.rotation += angle
+            p = np.matrix([frame.transform.translation[0] - pivot[0], frame.transform.translation[1] - pivot[1]]).T
+            rotation_mat = np.identity(2)
+            _cos = np.cos(angle / 180.0 * np.pi)
+            _sin = np.sin(angle / 180.0 * np.pi)
+            rotation_mat[0, 0] = _cos
+            rotation_mat[1, 0] = _sin
+            rotation_mat[0, 1] = -_sin
+            rotation_mat[1, 1] = _cos
+            delta_translation = rotation_mat * p - p
+            delta_translation = [float(delta_translation[0]), float(delta_translation[1])]
+            frame.transform.translation[0] += delta_translation[0]
+            frame.transform.translation[1] += delta_translation[1]
 
     def pivoted_scale(self, pivot, scale):
-        offset = [pivot[0] - self.transform.translation[0], pivot[1] - self.transform.translation[1]]
-        self.transform.translation[0] += offset[0] * (1.0 - scale)
-        self.transform.translation[1] += offset[1] * (1.0 - scale)
-        self.pixel_size *= scale
+        for frame in self.list_all_children(include_self=True):
+            offset = [pivot[0] - frame.transform.translation[0], pivot[1] - frame.transform.translation[1]]
+            frame.transform.translation[0] += offset[0] * (1.0 - scale)
+            frame.transform.translation[1] += offset[1] * (1.0 - scale)
+            frame.pivot_point[0] += offset[0] * (1.0 - scale)
+            frame.pivot_point[1] += offset[1] * (1.0 - scale)
+            frame.pixel_size *= scale
 
     def update_model_matrix(self):
         self.transform.scale = self.pixel_size
@@ -225,7 +241,7 @@ class CLEMFrame:
     def toggle_interpolation(self):
         self.interpolate = not self.interpolate
         if self.interpolate:
-            self.texture.set_linear_interpolation()
+            self.texture.set_linear_mipmap_interpolation()
         else:
             self.texture.set_no_interpolation()
 
@@ -265,18 +281,18 @@ class CLEMFrame:
             # ignore very bright pixels
             mean = np.mean(self.data)
             std = np.std(self.data)
-            self.hist_vals, self.hist_bins = np.histogram(self.data[self.data < (mean + 10 * std)], bins=CLEMFrame.HISTOGRAM_BINS)
+            self.hist_vals, self.hist_bins = np.histogram(self.data[self.data < (mean + 20 * std)], bins=CLEMFrame.HISTOGRAM_BINS)
 
             self.hist_vals = self.hist_vals.astype('float32')
             self.hist_bins = self.hist_bins.astype('float32')
-            self.hist_vals = np.delete(self.hist_vals, 0)
-            self.hist_bins = np.delete(self.hist_bins, 0)
+            #self.hist_vals = np.delete(self.hist_vals, 0)
+            #self.hist_bins = np.delete(self.hist_bins, 0)
             self.hist_vals = np.log(self.hist_vals + 1)
         else:
             for i in range(3):
                 vals, _ = np.histogram(self.data[:, :, i], bins=CLEMFrame.HISTOGRAM_BINS)
                 vals = vals.astype('float32')
-                vals = np.delete(vals, 0)
+                #vals = np.delete(vals, 0)
                 self.rgb_hist_vals.append(vals)
 
     def generate_va(self):
@@ -306,7 +322,7 @@ class CLEMFrame:
                              w, -h,
                              -w, -h]
         indices = [0, 1, 1, 2, 2, 3, 3, 0]
-        self.border_va.update(VertexBuffer(vertex_attributes), IndexBuffer(border_indices))
+        self.border_va.update(VertexBuffer(vertex_attributes), IndexBuffer(indices))
 
     def move_to_front(self):
         cfg.ce_frames.insert(0, cfg.ce_frames.pop(cfg.ce_frames.index(self)))
@@ -339,6 +355,7 @@ class Transform:
         self.rotation = 0.0
         self.scale = 1.0
         self.matrix = np.identity(4)
+        self.matrix_no_scale = np.identity(4)
 
     def compute_matrix(self):
         scale_mat = np.identity(4) * self.scale
@@ -357,6 +374,7 @@ class Transform:
         translation_mat[1, 3] = self.translation[1]
 
         self.matrix = np.matmul(translation_mat, np.matmul(rotation_mat, scale_mat))
+        self.matrix_no_scale = np.matmul(translation_mat, rotation_mat)
 
     def __add__(self, other):
         out = Transform()
@@ -374,3 +392,20 @@ class Transform:
         out.translation[1] = self.translation[1] - other.translation[1]
         out.rotation = self.rotation - other.rotation
         return out
+
+
+class ParticleGroup:
+    idgen = count(0)
+
+    def __init__(self, name="New group"):
+        self.id = next(ParticleGroup.idgen)
+        self.name = name
+        self.colour = (0.1, 0.2, 1.0, 1.0)
+        self.diameter = 200.0  # Angstrom
+        self.coordinates = list()
+
+    def __eq__(self, other):
+        if isinstance(other, ParticleGroup):
+            return self.id == other.id
+        return False
+
